@@ -76,7 +76,10 @@
  *       N_("Output tags (also known as metadata)"), NULL},
  *   {NULL}
  *  };
- *  ctx = g_option_context_new ("gst-launch");
+ *  // must initialise the threading system before using any other GLib funtion
+ *  if (!g_thread_supported ())
+ *    g_thread_init (NULL);
+ *  ctx = g_option_context_new ("[ADDITIONAL ARGUMENTS]");
  *  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
  *  g_option_context_add_group (ctx, gst_init_get_option_group ());
  *  if (!g_option_context_parse (ctx, &amp;argc, &amp;argv, &amp;err)) {
@@ -107,6 +110,9 @@
 #ifdef HAVE_FORK
 #include <sys/wait.h>
 #endif /* HAVE_FORK */
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -276,8 +282,11 @@ parse_debug_list (const gchar * list)
  * This function is useful if you want to integrate GStreamer with other
  * libraries that use GOption (see g_option_context_add_group() ).
  *
- * Returns: a pointer to GStreamer's option group. Should be dereferenced
- * after use.
+ * If you use this function, you should make sure you initialise the GLib
+ * threading system as one of the very first things in your program
+ * (see the example at the beginning of this section).
+ *
+ * Returns: a pointer to GStreamer's option group.
  */
 
 GOptionGroup *
@@ -334,6 +343,22 @@ gst_init_get_option_group (void)
     {NULL}
   };
 
+  /* The GLib threading system must be initialised before calling any other
+   * GLib function according to the documentation; if the application hasn't
+   * called gst_init() yet or initialised the threading system otherwise, we
+   * better issue a warning here (since chances are high that the application
+   * has already called other GLib functions such as g_option_context_new() */
+  if (!g_thread_supported ()) {
+    g_warning ("The GStreamer function gst_init_get_option_group() was\n"
+        "\tcalled, but the GLib threading system has not been initialised\n"
+        "\tyet, something that must happen before any other GLib function\n"
+        "\tis called. The application needs to be fixed so that it calls\n"
+        "\t   if (!g_thread_supported ()) g_thread_init(NULL);\n"
+        "\tas very first thing in its main() function. Please file a bug\n"
+        "\tagainst this application.");
+    g_thread_init (NULL);
+  }
+
   group = g_option_group_new ("gst", _("GStreamer Options"),
       _("Show GStreamer Options"), NULL, NULL);
   g_option_group_set_parse_hooks (group, (GOptionParseFunc) init_pre,
@@ -358,6 +383,10 @@ gst_init_get_option_group (void)
  * for some reason.  If you want your program to fail fatally,
  * use gst_init() instead.
  *
+ * This function should be called before calling any other GLib functions. If
+ * this is not an option, your program must initialise the GLib thread system
+ * using g_thread_init() before any other GLib functions are called.
+ *
  * Returns: %TRUE if GStreamer could be initialized.
  */
 gboolean
@@ -367,6 +396,9 @@ gst_init_check (int *argc, char **argv[], GError ** err)
   GOptionContext *ctx;
   gboolean res;
 
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
+
   GST_INFO ("initializing GStreamer");
 
   if (gst_initialized) {
@@ -400,6 +432,10 @@ gst_init_check (int *argc, char **argv[], GError ** err)
  * Initializes the GStreamer library, setting up internal path lists,
  * registering built-in elements, and loading standard plugins.
  *
+ * This function should be called before calling any other GLib functions. If
+ * this is not an option, your program must initialise the GLib thread system
+ * using g_thread_init() before any other GLib functions are called.
+ *
  * <note><para>
  * This function will terminate your program if it was unable to initialize
  * GStreamer for some reason.  If you want your program to fall back,
@@ -465,8 +501,6 @@ load_plugin_func (gpointer data, gpointer user_data)
       GST_WARNING ("Failed to load plugin: \"%s\"", filename);
     }
   }
-
-  g_free (data);
 }
 
 static void
@@ -499,6 +533,11 @@ static gboolean
 init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
     GError ** error)
 {
+  if (gst_initialized) {
+    GST_DEBUG ("already initialized");
+    return TRUE;
+  }
+
   /* GStreamer was built against a GLib >= 2.8 and is therefore not doing
    * the refcount hack. Check that it isn't being run against an older GLib */
   if (glib_major_version < 2 ||
@@ -511,12 +550,8 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
 
   g_type_init ();
 
-  if (g_thread_supported ()) {
-    /* somebody already initialized threading */
-  } else {
-    g_thread_init (NULL);
-  }
   /* we need threading to be enabled right here */
+  g_assert (g_thread_supported ());
   _gst_debug_init ();
 
 #ifdef ENABLE_NLS
@@ -542,6 +577,19 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
   GST_INFO ("Initializing GStreamer Core Library version %s", VERSION);
   GST_INFO ("Using library installed in %s", LIBDIR);
 
+  /* Print some basic system details if possible (OS/architecture) */
+#ifdef HAVE_SYS_UTSNAME_H
+  {
+    struct utsname sys_details;
+
+    if (uname (&sys_details) == 0) {
+      GST_INFO ("%s %s %s %s %s", sys_details.sysname,
+          sys_details.nodename, sys_details.release, sys_details.version,
+          sys_details.machine);
+    }
+  }
+#endif
+
   return TRUE;
 }
 
@@ -582,7 +630,7 @@ static GstPluginDesc plugin_desc = {
  * @registry_file: registry filename
  * @write_changes: write registry if it has changed?
  *
- * Scans for registry changes and evntualy updates the registry cache. 
+ * Scans for registry changes and eventually updates the registry cache. 
  *
  * Return: %TRUE if the registry could be updated
  */
@@ -595,18 +643,19 @@ scan_and_update_registry (GstRegistry * default_registry,
   GList *l;
 
   GST_DEBUG ("reading registry cache: %s", registry_file);
+#ifdef USE_BINARY_REGISTRY
+  gst_registry_binary_read_cache (default_registry, registry_file);
+#else
   gst_registry_xml_read_cache (default_registry, registry_file);
-
+#endif
   /* scan paths specified via --gst-plugin-path */
   GST_DEBUG ("scanning paths added via --gst-plugin-path");
   for (l = plugin_paths; l != NULL; l = l->next) {
     GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
     /* CHECKME: add changed |= here as well? */
     gst_registry_scan_path (default_registry, (gchar *) l->data);
-    g_free (l->data);
   }
-  g_list_free (plugin_paths);
-  plugin_paths = NULL;
+  /* keep plugin_paths around in case a re-scan is forced later on */
 
   /* GST_PLUGIN_PATH specifies a list of directories to scan for
    * additional plugins.  These take precedence over the system plugins */
@@ -669,7 +718,11 @@ scan_and_update_registry (GstRegistry * default_registry,
   }
 
   GST_INFO ("Registry cache changed. Writing new registry cache");
+#ifdef USE_BINARY_REGISTRY
+  if (!gst_registry_binary_write_cache (default_registry, registry_file)) {
+#else
   if (!gst_registry_xml_write_cache (default_registry, registry_file)) {
+#endif
     g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
         _("Error writing registry cache to %s: %s"),
         registry_file, g_strerror (errno));
@@ -699,6 +752,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 #ifdef HAVE_FORK
   pid_t pid;
   int pfd[2];
+  int ret;
 
   /* We fork here, and let the child read and possibly rebuild the registry.
    * After that, the parent will re-read the freshly generated registry. */
@@ -725,7 +779,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
     gchar res_byte;
 
     /* this is the child. Close the read pipe */
-    close (pfd[0]);
+    (void) close (pfd[0]);
 
     GST_DEBUG ("child reading registry cache");
     res =
@@ -740,20 +794,26 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 
     /* write a result byte to the pipe */
     res_byte = res ? '1' : '0';
-    if (write (pfd[1], &res_byte, 1) != 1 || close (pfd[1]) != 0) {
-      /* could not write to pipe, probably means parent has exited before us */
-    }
+    do {
+      ret = write (pfd[1], &res_byte, 1);
+    } while (ret == -1 && errno == EINTR);
+    /* if ret == -1 now, we could not write to pipe, probably 
+     * means parent has exited before us */
+    (void) close (pfd[1]);
+
     _exit (0);
   } else {
-    int ret;
     gchar res_byte;
 
     /* parent. Close write pipe */
-    close (pfd[1]);
+    (void) close (pfd[1]);
 
     /* Wait for result from the pipe */
     GST_DEBUG ("Waiting for data from child");
-    ret = read (pfd[0], &res_byte, 1);
+    do {
+      ret = read (pfd[0], &res_byte, 1);
+    } while (ret == -1 && errno == EINTR);
+
     if (ret == -1) {
       g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
           _("Error re-scanning registry %s: %s"),
@@ -761,7 +821,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
       close (pfd[0]);
       return FALSE;
     }
-    close (pfd[0]);
+    (void) close (pfd[0]);
 
     /* Wait to ensure the child is reaped, but ignore the result */
     GST_DEBUG ("parent waiting on child");
@@ -777,7 +837,11 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 
     if (res_byte == '1') {
       GST_DEBUG ("Child succeeded. Parent reading registry cache");
+#ifdef USE_BINARY_REGISTRY
+      gst_registry_binary_read_cache (default_registry, registry_file);
+#else
       gst_registry_xml_read_cache (default_registry, registry_file);
+#endif
     } else {
       GST_DEBUG ("Child failed. Parent re-scanning registry, ignoring errors.");
       scan_and_update_registry (default_registry, registry_file, FALSE, NULL);
@@ -798,8 +862,13 @@ ensure_current_registry (GError ** error)
   default_registry = gst_registry_get_default ();
   registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
   if (registry_file == NULL) {
+#ifdef USE_BINARY_REGISTRY
+    registry_file = g_build_filename (g_get_home_dir (),
+        ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".bin", NULL);
+#else
     registry_file = g_build_filename (g_get_home_dir (),
         ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".xml", NULL);
+#endif
   }
 
   /* first see if forking is enabled */
@@ -852,6 +921,11 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   GstTrace *gst_trace;
 #endif /* GST_DISABLE_TRACE */
 
+  if (gst_initialized) {
+    GST_DEBUG ("already initialized");
+    return TRUE;
+  }
+
   llf = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL;
   g_log_set_handler (g_log_domain_gstreamer, llf, debug_log_handler, NULL);
 
@@ -901,12 +975,10 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
     return FALSE;
 #endif /* GST_DISABLE_REGISTRY */
 
-  /* if we need to preload plugins */
-  if (preload_plugins) {
-    g_slist_foreach (preload_plugins, load_plugin_func, NULL);
-    g_slist_free (preload_plugins);
-    preload_plugins = NULL;
-  }
+  /* if we need to preload plugins, do so now */
+  g_slist_foreach (preload_plugins, load_plugin_func, NULL);
+  /* keep preload_plugins around in case a re-scan is forced later on */
+
 #ifndef GST_DISABLE_TRACE
   _gst_trace_on = 0;
   if (_gst_trace_on) {
@@ -931,6 +1003,7 @@ sort_by_category_name (gconstpointer a, gconstpointer b)
   return strcmp (gst_debug_category_get_name ((GstDebugCategory *) a),
       gst_debug_category_get_name ((GstDebugCategory *) b));
 }
+
 static void
 gst_debug_help (void)
 {
@@ -1114,6 +1187,14 @@ gst_deinit (void)
     return;
   }
 
+  g_slist_foreach (preload_plugins, (GFunc) g_free, NULL);
+  g_slist_free (preload_plugins);
+  preload_plugins = NULL;
+
+  g_list_foreach (plugin_paths, (GFunc) g_free, NULL);
+  g_list_free (plugin_paths);
+  plugin_paths = NULL;
+
   clock = gst_system_clock_obtain ();
   gst_object_unref (clock);
   gst_object_unref (clock);
@@ -1247,3 +1328,55 @@ gst_registry_fork_set_enabled (gboolean enabled)
   _gst_enable_registry_fork = enabled;
 #endif /* HAVE_FORK */
 }
+
+
+/**
+ * gst_update_registry:
+ *
+ * Forces GStreamer to re-scan its plugin paths and update the default
+ * plugin registry.
+ *
+ * Applications will almost never need to call this function, it is only
+ * useful if the application knows new plugins have been installed (or old
+ * ones removed) since the start of the application (or, to be precise, the
+ * first call to gst_init()) and the application wants to make use of any
+ * newly-installed plugins without restarting the application.
+ *
+ * Applications should assume that the registry update is neither atomic nor
+ * thread-safe and should therefore not have any dynamic pipelines running
+ * (including the playbin and decodebin elements) and should also not create
+ * any elements or access the GStreamer registry while the update is in
+ * progress.
+ *
+ * Note that this function may block for a significant amount of time.
+ *
+ * Returns: %TRUE if the registry has been updated successfully (does not
+ *          imply that there were changes), otherwise %FALSE.
+ *
+ * Since: 0.10.12
+ */
+gboolean
+gst_update_registry (void)
+{
+  gboolean res = FALSE;
+
+#ifndef GST_DISABLE_REGISTRY
+  GError *err = NULL;
+
+  res = ensure_current_registry (&err);
+  if (err) {
+    GST_WARNING ("registry update failed: %s", err->message);
+    g_error_free (err);
+  } else {
+    GST_LOG ("registry update succeeded");
+  }
+
+  if (preload_plugins) {
+    g_slist_foreach (preload_plugins, load_plugin_func, NULL);
+  }
+#else
+  GST_WARNING ("registry update failed: %s", "registry disabled");
+#endif /* GST_DISABLE_REGISTRY */
+
+  return res;
+}
