@@ -53,6 +53,10 @@
  * Alternatively, use gst_buffer_new_and_alloc()
  * to create a buffer with preallocated data of a given size.
  *
+ * The data pointed to by the buffer can be retrieved with the GST_BUFFER_DATA()
+ * macro. The size of the data can be found with GST_BUFFER_SIZE(). For buffers
+ * of size 0, the data pointer is undefined (usually NULL) and should never be used.
+ *
  * If an element knows what pad you will push the buffer out on, it should use
  * gst_pad_alloc_buffer() instead to create a buffer.  This allows downstream
  * elements to provide special buffers to write in, like hardware buffers.
@@ -107,7 +111,7 @@
  * the refcount drops to 0, any data pointed to by GST_BUFFER_MALLOCDATA() will
  * also be freed.
  *
- * Last reviewed on November 23th, 2005 (0.9.5)
+ * Last reviewed on August 11th, 2006 (0.10.10)
  */
 #include "gst_private.h"
 
@@ -116,13 +120,15 @@
 #include "gstutils.h"
 #include "gstminiobject.h"
 
-
 static void gst_buffer_init (GTypeInstance * instance, gpointer g_class);
 static void gst_buffer_class_init (gpointer g_class, gpointer class_data);
 static void gst_buffer_finalize (GstBuffer * buffer);
 static GstBuffer *_gst_buffer_copy (GstBuffer * buffer);
 static GType gst_subbuffer_get_type (void);
 
+static GType _gst_subbuffer_type = 0;
+static GType _gst_buffer_type = 0;
+
 void
 _gst_buffer_initialize (void)
 {
@@ -141,8 +147,6 @@ _gst_buffer_initialize (void)
 GType
 gst_buffer_get_type (void)
 {
-  static GType _gst_buffer_type;
-
   if (G_UNLIKELY (_gst_buffer_type == 0)) {
     static const GTypeInfo buffer_info = {
       sizeof (GstBufferClass),
@@ -183,9 +187,7 @@ gst_buffer_finalize (GstBuffer * buffer)
   GST_CAT_LOG (GST_CAT_BUFFER, "finalize %p", buffer);
 
   /* free our data */
-  if (buffer->malloc_data) {
-    g_free (buffer->malloc_data);
-  }
+  g_free (buffer->malloc_data);
 
   gst_caps_replace (&GST_BUFFER_CAPS (buffer), NULL);
 }
@@ -207,12 +209,12 @@ _gst_buffer_copy (GstBuffer * buffer)
   mask = GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_IN_CAPS |
       GST_BUFFER_FLAG_DELTA_UNIT | GST_BUFFER_FLAG_DISCONT |
       GST_BUFFER_FLAG_GAP;
-  GST_MINI_OBJECT (copy)->flags |= GST_MINI_OBJECT (buffer)->flags & mask;
+  GST_MINI_OBJECT_FLAGS (copy) |= GST_MINI_OBJECT_FLAGS (buffer) & mask;
 
   /* we simply copy everything from our parent */
   copy->data = g_memdup (buffer->data, buffer->size);
   /* make sure it gets freed (even if the parent is subclassed, we return a
-     normal buffer */
+     normal buffer) */
   copy->malloc_data = copy->data;
 
   copy->size = buffer->size;
@@ -258,7 +260,7 @@ gst_buffer_new (void)
 {
   GstBuffer *newbuf;
 
-  newbuf = (GstBuffer *) gst_mini_object_new (GST_TYPE_BUFFER);
+  newbuf = (GstBuffer *) gst_mini_object_new (_gst_buffer_type);
 
   GST_CAT_LOG (GST_CAT_BUFFER, "new %p", newbuf);
 
@@ -272,6 +274,8 @@ gst_buffer_new (void)
  * Creates a newly allocated buffer with data of the given size.
  * The buffer memory is not cleared.
  *
+ * Note that when @size == 0, the buffer data pointer will be NULL.
+ *
  * MT safe.
  * Returns: the new #GstBuffer.
  */
@@ -352,7 +356,7 @@ gst_buffer_set_caps (GstBuffer * buffer, GstCaps * caps)
 gboolean
 gst_buffer_is_metadata_writable (GstBuffer * buf)
 {
-  return (GST_MINI_OBJECT_REFCOUNT_VALUE (GST_MINI_OBJECT (buf)) == 1);
+  return (GST_MINI_OBJECT_REFCOUNT_VALUE (GST_MINI_OBJECT_CAST (buf)) == 1);
 }
 
 /**
@@ -364,7 +368,8 @@ gst_buffer_is_metadata_writable (GstBuffer * buf)
  * is solely owned by the caller, by creating a subbuffer of the original
  * buffer if necessary.
  * 
- * After calling this function, @buf should not be referenced anymore.
+ * After calling this function, @buf should not be referenced anymore. The
+ * result of this function has guaranteed writable metadata.
  *
  * Returns: A new #GstBuffer with writable metadata.
  */
@@ -377,8 +382,13 @@ gst_buffer_make_metadata_writable (GstBuffer * buf)
     ret = buf;
   } else {
     ret = gst_buffer_create_sub (buf, 0, GST_BUFFER_SIZE (buf));
+
+    /* copy all the flags except IN_CAPS */
     GST_BUFFER_FLAGS (ret) = GST_BUFFER_FLAGS (buf);
     GST_BUFFER_FLAG_UNSET (ret, GST_BUFFER_FLAG_IN_CAPS);
+    /* data should always be set to READONLY */
+    GST_BUFFER_FLAG_SET (ret, GST_BUFFER_FLAG_READONLY);
+
     gst_buffer_unref (buf);
   }
 
@@ -388,10 +398,8 @@ gst_buffer_make_metadata_writable (GstBuffer * buf)
 typedef struct _GstSubBuffer GstSubBuffer;
 typedef struct _GstSubBufferClass GstSubBufferClass;
 
-#define GST_TYPE_SUBBUFFER                         (gst_subbuffer_get_type())
-
-#define GST_IS_SUBBUFFER(obj)  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_SUBBUFFER))
-#define GST_SUBBUFFER(obj)     (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_SUBBUFFER, GstSubBuffer))
+#define GST_IS_SUBBUFFER(obj)   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), _gst_subbuffer_type))
+#define GST_SUBBUFFER_CAST(obj) ((GstSubBuffer *)(obj))
 
 struct _GstSubBuffer
 {
@@ -414,8 +422,6 @@ static void gst_subbuffer_finalize (GstSubBuffer * buffer);
 static GType
 gst_subbuffer_get_type (void)
 {
-  static GType _gst_subbuffer_type = 0;
-
   if (G_UNLIKELY (_gst_subbuffer_type == 0)) {
     static const GTypeInfo subbuffer_info = {
       sizeof (GstSubBufferClass),
@@ -452,7 +458,8 @@ gst_subbuffer_finalize (GstSubBuffer * buffer)
 {
   gst_buffer_unref (buffer->parent);
 
-  GST_MINI_OBJECT_CLASS (sub_parent_class)->finalize (GST_MINI_OBJECT (buffer));
+  GST_MINI_OBJECT_CLASS (sub_parent_class)->
+      finalize (GST_MINI_OBJECT_CAST (buffer));
 }
 
 static void
@@ -466,13 +473,16 @@ gst_subbuffer_init (GTypeInstance * instance, gpointer g_class)
  * @parent: a #GstBuffer.
  * @offset: the offset into parent #GstBuffer at which the new sub-buffer 
  *          begins.
- * @size: the size of the new #GstBuffer sub-buffer, in bytes (with size > 0).
+ * @size: the size of the new #GstBuffer sub-buffer, in bytes.
  *
  * Creates a sub-buffer from @parent at @offset and @size.
  * This sub-buffer uses the actual memory space of the parent buffer.
  * This function will copy the offset and timestamp fields when the
- * offset is 0, else they are set to #GST_CLOCK_TIME_NONE/#GST_BUFFER_OFFSET_NONE.
- * The duration field of the new buffer is set to #GST_CLOCK_TIME_NONE.
+ * offset is 0. If not, they will be set to #GST_CLOCK_TIME_NONE and 
+ * #GST_BUFFER_OFFSET_NONE.
+ * If @offset equals 0 and @size equals the total size of @buffer, the
+ * duration and offset end fields are also copied. If not they will be set
+ * to #GST_CLOCK_TIME_NONE and #GST_BUFFER_OFFSET_NONE.
  *
  * MT safe.
  * Returns: the new #GstBuffer.
@@ -483,22 +493,22 @@ gst_buffer_create_sub (GstBuffer * buffer, guint offset, guint size)
 {
   GstSubBuffer *subbuffer;
   GstBuffer *parent;
+  gboolean complete;
 
   g_return_val_if_fail (buffer != NULL, NULL);
   g_return_val_if_fail (buffer->mini_object.refcount > 0, NULL);
-  g_return_val_if_fail (size > 0, NULL);
   g_return_val_if_fail (buffer->size >= offset + size, NULL);
 
   /* find real parent */
   if (GST_IS_SUBBUFFER (buffer)) {
-    parent = GST_SUBBUFFER (buffer)->parent;
+    parent = GST_SUBBUFFER_CAST (buffer)->parent;
   } else {
     parent = buffer;
   }
   gst_buffer_ref (parent);
 
   /* create the new buffer */
-  subbuffer = (GstSubBuffer *) gst_mini_object_new (GST_TYPE_SUBBUFFER);
+  subbuffer = (GstSubBuffer *) gst_mini_object_new (_gst_subbuffer_type);
   subbuffer->parent = parent;
 
   GST_CAT_LOG (GST_CAT_BUFFER, "new subbuffer %p (parent %p)", subbuffer,
@@ -513,16 +523,28 @@ gst_buffer_create_sub (GstBuffer * buffer, guint offset, guint size)
   if (offset == 0) {
     GST_BUFFER_TIMESTAMP (subbuffer) = GST_BUFFER_TIMESTAMP (buffer);
     GST_BUFFER_OFFSET (subbuffer) = GST_BUFFER_OFFSET (buffer);
+    complete = (buffer->size == size);
   } else {
     GST_BUFFER_TIMESTAMP (subbuffer) = GST_CLOCK_TIME_NONE;
     GST_BUFFER_OFFSET (subbuffer) = GST_BUFFER_OFFSET_NONE;
+    complete = FALSE;
   }
 
-  GST_BUFFER_DURATION (subbuffer) = GST_CLOCK_TIME_NONE;
-  GST_BUFFER_OFFSET_END (subbuffer) = GST_BUFFER_OFFSET_NONE;
-
-  GST_BUFFER_CAPS (subbuffer) = NULL;
+  if (complete) {
+    GstCaps *caps;
 
+    /* if we copied the complete buffer we can copy the duration,
+     * offset_end and caps as well */
+    GST_BUFFER_DURATION (subbuffer) = GST_BUFFER_DURATION (buffer);
+    GST_BUFFER_OFFSET_END (subbuffer) = GST_BUFFER_OFFSET_END (buffer);
+    if ((caps = GST_BUFFER_CAPS (buffer)))
+      gst_caps_ref (caps);
+    GST_BUFFER_CAPS (subbuffer) = caps;
+  } else {
+    GST_BUFFER_DURATION (subbuffer) = GST_CLOCK_TIME_NONE;
+    GST_BUFFER_OFFSET_END (subbuffer) = GST_BUFFER_OFFSET_NONE;
+    GST_BUFFER_CAPS (subbuffer) = NULL;
+  }
   return GST_BUFFER_CAST (subbuffer);
 }
 
@@ -549,8 +571,8 @@ gst_buffer_is_span_fast (GstBuffer * buf1, GstBuffer * buf2)
   /* it's only fast if we have subbuffers of the same parent */
   return (GST_IS_SUBBUFFER (buf1) &&
       GST_IS_SUBBUFFER (buf2) &&
-      (GST_SUBBUFFER (buf1)->parent == GST_SUBBUFFER (buf2)->parent) &&
-      ((buf1->data + buf1->size) == buf2->data));
+      (GST_SUBBUFFER_CAST (buf1)->parent == GST_SUBBUFFER_CAST (buf2)->parent)
+      && ((buf1->data + buf1->size) == buf2->data));
 }
 
 /**
@@ -589,7 +611,7 @@ gst_buffer_span (GstBuffer * buf1, guint32 offset, GstBuffer * buf2,
 
   /* if the two buffers have the same parent and are adjacent */
   if (gst_buffer_is_span_fast (buf1, buf2)) {
-    GstBuffer *parent = GST_SUBBUFFER (buf1)->parent;
+    GstBuffer *parent = GST_SUBBUFFER_CAST (buf1)->parent;
 
     /* we simply create a subbuffer of the common parent */
     newbuf = gst_buffer_create_sub (parent,
