@@ -128,6 +128,11 @@ static GstStateChangeReturn gst_element_set_state_func (GstElement * element,
     GstState state);
 static void gst_element_set_bus_func (GstElement * element, GstBus * bus);
 
+static gboolean gst_element_default_send_event (GstElement * element,
+    GstEvent * event);
+static gboolean gst_element_default_query (GstElement * element,
+    GstQuery * query);
+
 #ifndef GST_DISABLE_LOADSAVE
 static xmlNodePtr gst_element_save_thyself (GstObject * object,
     xmlNodePtr parent);
@@ -219,6 +224,8 @@ gst_element_class_init (GstElementClass * klass)
   klass->set_state = GST_DEBUG_FUNCPTR (gst_element_set_state_func);
   klass->get_state = GST_DEBUG_FUNCPTR (gst_element_get_state_func);
   klass->set_bus = GST_DEBUG_FUNCPTR (gst_element_set_bus_func);
+  klass->query = GST_DEBUG_FUNCPTR (gst_element_default_query);
+  klass->send_event = GST_DEBUG_FUNCPTR (gst_element_default_send_event);
   klass->numpadtemplates = 0;
 
   klass->elementfactory = NULL;
@@ -261,7 +268,7 @@ gst_element_init (GstElement * element)
  * @object: a #GObject that signalled the error.
  * @orig: the #GstObject that initiated the error.
  * @error: the GError.
- * @debug: an additional debug information string, or NULL.
+ * @debug: an additional debug information string, or %NULL.
  *
  * A default error signal callback to attach to an element.
  * The user data passed to the g_signal_connect is ignored.
@@ -367,7 +374,7 @@ gst_element_provides_clock (GstElement * element)
  * <note>An element is only required to provide a clock in the PAUSED
  * state. Some elements can provide a clock in other states.</note>
  *
- * Returns: the GstClock provided by the element or NULL
+ * Returns: the GstClock provided by the element or %NULL
  * if no clock could be provided.  Unref after usage.
  *
  * MT safe.
@@ -557,7 +564,7 @@ gst_element_set_index (GstElement * element, GstIndex * index)
  *
  * Gets the index from the element.
  *
- * Returns: a #GstIndex or NULL when no index was set on the
+ * Returns: a #GstIndex or %NULL when no index was set on the
  * element. unref after usage.
  *
  * MT safe.
@@ -605,6 +612,7 @@ gboolean
 gst_element_add_pad (GstElement * element, GstPad * pad)
 {
   gchar *pad_name;
+  gboolean flushing;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
   g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
@@ -614,6 +622,7 @@ gst_element_add_pad (GstElement * element, GstPad * pad)
   pad_name = g_strdup (GST_PAD_NAME (pad));
   GST_CAT_INFO_OBJECT (GST_CAT_ELEMENT_PADS, element, "adding pad '%s'",
       GST_STR_NULL (pad_name));
+  flushing = GST_PAD_IS_FLUSHING (pad);
   GST_OBJECT_UNLOCK (pad);
 
   /* then check to see if there's already a pad by that name here */
@@ -626,6 +635,17 @@ gst_element_add_pad (GstElement * element, GstPad * pad)
               GST_OBJECT_CAST (element))))
     goto had_parent;
 
+  /* check for flushing pads */
+  if (flushing && (GST_STATE (element) > GST_STATE_READY ||
+          GST_STATE_NEXT (element) == GST_STATE_PAUSED)) {
+    g_warning ("adding flushing pad '%s' to running element '%s'",
+        GST_STR_NULL (pad_name), GST_ELEMENT_NAME (element));
+    /* unset flushing */
+    GST_OBJECT_LOCK (pad);
+    GST_PAD_UNSET_FLUSHING (pad);
+    GST_OBJECT_UNLOCK (pad);
+  }
+
   g_free (pad_name);
 
   /* add it to the list */
@@ -695,8 +715,9 @@ no_direction:
  * gst_element_release_request_pad() function instead.
  *
  * Pads are not automatically deactivated so elements should perform the needed
- * steps to deactivate the pad in case this pad is removed in the PAUSED or PLAYING
- * state. See gst_pad_set_active() for more information about deactivating pads.
+ * steps to deactivate the pad in case this pad is removed in the PAUSED or
+ * PLAYING state. See gst_pad_set_active() for more information about
+ * deactivating pads.
  *
  * The pad and the element should be unlocked when calling this function.
  *
@@ -819,7 +840,7 @@ pad_compare_name (GstPad * pad1, const gchar * name)
  * Retrieves a pad from @element by name. This version only retrieves
  * already-existing (i.e. 'static') pads.
  *
- * Returns: the requested #GstPad if found, otherwise NULL. unref after
+ * Returns: the requested #GstPad if found, otherwise %NULL. unref after
  * usage.
  *
  * MT safe.
@@ -880,7 +901,7 @@ gst_element_request_pad (GstElement * element, GstPadTemplate * templ,
  * request pads. The pad should be released with 
  * gst_element_release_request_pad().
  *
- * Returns: requested #GstPad if found, otherwise NULL. Release after usage.
+ * Returns: requested #GstPad if found, otherwise %NULL. Release after usage.
  */
 GstPad *
 gst_element_get_request_pad (GstElement * element, const gchar * name)
@@ -1133,7 +1154,7 @@ gst_element_class_get_pad_template_list (GstElementClass * element_class)
  * that has subclasses, make sure to pass the g_class parameter of the
  * #GInstanceInitFunc here.</note>
  *
- * Returns: the #GstPadTemplate with the given name, or NULL if none was found.
+ * Returns: the #GstPadTemplate with the given name, or %NULL if none was found.
  * No unreferencing is necessary.
  */
 GstPadTemplate *
@@ -1221,6 +1242,32 @@ wrong_direction:
   }
 }
 
+static gboolean
+gst_element_default_send_event (GstElement * element, GstEvent * event)
+{
+  gboolean result = FALSE;
+  GstPad *pad;
+
+  pad = GST_EVENT_IS_DOWNSTREAM (event) ?
+      gst_element_get_random_pad (element, TRUE, GST_PAD_SRC) :
+      gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
+
+  if (pad) {
+    GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS,
+        "pushing %s event to random %s pad %s:%s",
+        GST_EVENT_TYPE_NAME (event),
+        (GST_PAD_DIRECTION (pad) == GST_PAD_SRC ? "src" : "sink"),
+        GST_DEBUG_PAD_NAME (pad));
+
+    result = gst_pad_push_event (pad, event);
+    gst_object_unref (pad);
+  } else {
+    GST_CAT_INFO (GST_CAT_ELEMENT_PADS, "can't send %s event on element %s",
+        GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
+  }
+  return result;
+}
+
 /**
  * gst_element_send_event:
  * @element: a #GstElement to send the event to.
@@ -1253,23 +1300,7 @@ gst_element_send_event (GstElement * element, GstEvent * event)
         GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
     result = oclass->send_event (element, event);
   } else {
-    GstPad *pad = GST_EVENT_IS_DOWNSTREAM (event) ?
-        gst_element_get_random_pad (element, TRUE, GST_PAD_SRC) :
-        gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
-
-    if (pad) {
-      GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS,
-          "pushing %s event to random %s pad %s:%s",
-          GST_EVENT_TYPE_NAME (event),
-          (GST_PAD_DIRECTION (pad) == GST_PAD_SRC ? "src" : "sink"),
-          GST_DEBUG_PAD_NAME (pad));
-
-      result = gst_pad_push_event (pad, event);
-      gst_object_unref (pad);
-    } else {
-      GST_CAT_INFO (GST_CAT_ELEMENT_PADS, "can't send %s event on element %s",
-          GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
-    }
+    result = gst_element_default_send_event (element, event);
   }
   return result;
 }
@@ -1352,6 +1383,33 @@ gst_element_get_query_types (GstElement * element)
   return result;
 }
 
+static gboolean
+gst_element_default_query (GstElement * element, GstQuery * query)
+{
+  gboolean result = FALSE;
+  GstPad *pad;
+
+  pad = gst_element_get_random_pad (element, FALSE, GST_PAD_SRC);
+  if (pad) {
+    result = gst_pad_query (pad, query);
+
+    gst_object_unref (pad);
+  } else {
+    pad = gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
+    if (pad) {
+      GstPad *peer = gst_pad_get_peer (pad);
+
+      if (peer) {
+        result = gst_pad_query (peer, query);
+
+        gst_object_unref (peer);
+      }
+      gst_object_unref (pad);
+    }
+  }
+  return result;
+}
+
 /**
  * gst_element_query:
  * @element: a #GstElement to perform the query on.
@@ -1383,25 +1441,7 @@ gst_element_query (GstElement * element, GstQuery * query)
         GST_ELEMENT_NAME (element));
     result = oclass->query (element, query);
   } else {
-    GstPad *pad = gst_element_get_random_pad (element, FALSE, GST_PAD_SRC);
-
-    if (pad) {
-      result = gst_pad_query (pad, query);
-
-      gst_object_unref (pad);
-    } else {
-      pad = gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
-      if (pad) {
-        GstPad *peer = gst_pad_get_peer (pad);
-
-        if (peer) {
-          result = gst_pad_query (peer, query);
-
-          gst_object_unref (peer);
-        }
-        gst_object_unref (pad);
-      }
-    }
+    result = gst_element_default_query (element, query);
   }
   return result;
 }
@@ -1457,11 +1497,11 @@ no_bus:
 
 /**
  * _gst_element_error_printf:
- * @format: the printf-like format to use, or NULL
+ * @format: the printf-like format to use, or %NULL
  *
  * This function is only used internally by the gst_element_error() macro.
  *
- * Returns: a newly allocated string, or NULL if the format was NULL or ""
+ * Returns: a newly allocated string, or %NULL if the format was %NULL or ""
  *
  * MT safe.
  */
@@ -1489,14 +1529,17 @@ _gst_element_error_printf (const gchar * format, ...)
  * @domain:   the GStreamer GError domain this message belongs to
  * @code:     the GError code belonging to the domain
  * @text:     an allocated text string to be used as a replacement for the
- *            default message connected to code, or NULL
+ *            default message connected to code, or %NULL
  * @debug:    an allocated debug message to be used as a replacement for the
- *            default debugging information, or NULL
+ *            default debugging information, or %NULL
  * @file:     the source code file where the error was generated
  * @function: the source code function where the error was generated
  * @line:     the source code line where the error was generated
  *
- * Post an error or warning message on the bus from inside an element.
+ * Post an error, warning or info message on the bus from inside an element.
+ *
+ * @type must be of #GST_MESSAGE_ERROR, #GST_MESSAGE_WARNING or
+ * #GST_MESSAGE_INFO.
  *
  * MT safe.
  */
@@ -1516,7 +1559,7 @@ void gst_element_message_full
   GST_DEBUG_OBJECT (element, "start");
   g_return_if_fail (GST_IS_ELEMENT (element));
   g_return_if_fail ((type == GST_MESSAGE_ERROR) ||
-      (type == GST_MESSAGE_WARNING));
+      (type == GST_MESSAGE_WARNING) || (type == GST_MESSAGE_INFO));
 
   /* check if we send the given text or the default error text */
   if ((text == NULL) || (text[0] == 0)) {
@@ -1547,14 +1590,22 @@ void gst_element_message_full
       sent_text);
   gerror = g_error_new_literal (domain, code, sent_text);
 
-  if (type == GST_MESSAGE_ERROR) {
-    message =
-        gst_message_new_error (GST_OBJECT_CAST (element), gerror, sent_debug);
-  } else if (type == GST_MESSAGE_WARNING) {
-    message = gst_message_new_warning (GST_OBJECT_CAST (element), gerror,
-        sent_debug);
-  } else {
-    g_assert_not_reached ();
+  switch (type) {
+    case GST_MESSAGE_ERROR:
+      message =
+          gst_message_new_error (GST_OBJECT_CAST (element), gerror, sent_debug);
+      break;
+    case GST_MESSAGE_WARNING:
+      message = gst_message_new_warning (GST_OBJECT_CAST (element), gerror,
+          sent_debug);
+      break;
+    case GST_MESSAGE_INFO:
+      message = gst_message_new_info (GST_OBJECT_CAST (element), gerror,
+          sent_debug);
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
   }
   gst_element_post_message (element, message);
 
@@ -1605,7 +1656,7 @@ gst_element_is_locked_state (GstElement * element)
  *
  * MT safe.
  *
- * Returns: TRUE if the state was changed, FALSE if bad parameterss were given
+ * Returns: TRUE if the state was changed, FALSE if bad parameters were given
  * or the elements state-locking needed no change.
  */
 gboolean
@@ -1635,11 +1686,13 @@ gst_element_set_locked_state (GstElement * element, gboolean locked_state)
   return TRUE;
 
 was_ok:
-  GST_CAT_DEBUG (GST_CAT_STATES, "elements %s was in locked state %d",
-      GST_ELEMENT_NAME (element), old);
-  GST_OBJECT_UNLOCK (element);
+  {
+    GST_CAT_DEBUG (GST_CAT_STATES, "elements %s was in locked state %d",
+        GST_ELEMENT_NAME (element), old);
+    GST_OBJECT_UNLOCK (element);
 
-  return FALSE;
+    return FALSE;
+  }
 }
 
 /**
@@ -1701,10 +1754,13 @@ gst_element_get_state_func (GstElement * element,
   GstStateChangeReturn ret = GST_STATE_CHANGE_FAILURE;
   GstState old_pending;
 
-  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "getting state");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "getting state, timeout %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (timeout));
 
   GST_OBJECT_LOCK (element);
   ret = GST_STATE_RETURN (element);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "RETURN is %s",
+      gst_element_state_change_return_get_name (ret));
 
   /* we got an error, report immediatly */
   if (ret == GST_STATE_CHANGE_FAILURE)
@@ -1820,13 +1876,12 @@ interrupted:
  *
  * This function returns %GST_STATE_CHANGE_NO_PREROLL if the element
  * successfully changed its state but is not able to provide data yet.
- * This mostly
- * happens for live sources that only produce data in the PLAYING state.
- * While the state change return is equivalent to %GST_STATE_CHANGE_SUCCESS, it
- * is returned to the application to signal that some sink elements might not
- * be able to complete their state change because an element is not producing
- * data to complete the preroll. When setting the element to playing,
- * the preroll will complete and playback will start.
+ * This mostly happens for live sources that only produce data in the PLAYING
+ * state. While the state change return is equivalent to
+ * %GST_STATE_CHANGE_SUCCESS, it is returned to the application to signal that
+ * some sink elements might not be able to complete their state change because
+ * an element is not producing data to complete the preroll. When setting the
+ * element to playing, the preroll will complete and playback will start.
  *
  * Returns: %GST_STATE_CHANGE_SUCCESS if the element has no more pending state
  *          and the last state change succeeded, %GST_STATE_CHANGE_ASYNC if the
@@ -1915,13 +1970,13 @@ nothing_aborted:
  * by elements that do asynchronous state changes.       
  * The core will normally call this method automatically when an         
  * element returned %GST_STATE_CHANGE_SUCCESS from the state change function.      
- * Elements that return %GST_STATE_CHANGE_ASYNC from the change_state function
- * should eventually call this method from the streaming thread to signal       
- * successfull state change completion.          
- *       
+ *
  * If after calling this method the element still has not reached        
  * the pending state, the next state change is performed.        
  *       
+ * This method is used internally and should normally not be called by plugins
+ * or applications.
+ *       
  * Returns: The result of the commit state change.       
  *       
  * MT safe.      
@@ -2027,7 +2082,8 @@ complete:
  * This function can only be called when the element is currently
  * not in error or an async state change.
  *
- * This function can only be called with the STATE_LOCK held.
+ * This function is used internally and should normally not be called from
+ * plugins or applications.
  *
  * MT safe.
  */
@@ -2141,6 +2197,7 @@ gst_element_set_state_func (GstElement * element, GstState state)
   current = GST_STATE (element);
   next = GST_STATE_NEXT (element);
   old_pending = GST_STATE_PENDING (element);
+  /* increment state cookie so that we can track each state change */
   element->state_cookie++;
 
   /* this is the (new) state we should go to */
@@ -2295,6 +2352,7 @@ invalid_return:
     g_critical ("%s: unknown return value %d from a state change function",
         GST_ELEMENT_NAME (element), ret);
 
+    /* we are in error now */
     ret = GST_STATE_CHANGE_FAILURE;
     GST_STATE_RETURN (element) = ret;
     GST_OBJECT_UNLOCK (element);
