@@ -380,6 +380,7 @@ gst_object_sink (gpointer object)
 
   GST_OBJECT_LOCK (object);
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
+    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "clear floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
     gst_object_unref (object);
@@ -1142,7 +1143,7 @@ gst_signal_object_get_type (void)
 {
   static GType signal_object_type = 0;
 
-  if (!signal_object_type) {
+  if (G_UNLIKELY (signal_object_type == 0)) {
     static const GTypeInfo signal_object_info = {
       sizeof (GstSignalObjectClass),
       NULL,
