@@ -433,53 +433,62 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
       GstClockTime new_base_time;
+      GstClockTime start_time, stream_time, delay;
+      gboolean new_clock;
+
+      GST_DEBUG_OBJECT (element, "selecting clock and base_time");
 
       /* when going to playing, select a clock */
       clock = gst_element_provide_clock (element);
 
       if (clock) {
-        GstClockTime start_time, stream_time, delay;
-        gboolean new_clock;
-
         start_time = gst_clock_get_time (clock);
-
-        GST_OBJECT_LOCK (element);
-        new_clock = element->clock != clock;
-        stream_time = pipeline->stream_time;
-        delay = pipeline->delay;
-        GST_OBJECT_UNLOCK (element);
-
-        if (new_clock) {
-          /* now distribute the clock (which could be NULL). If some
-           * element refuses the clock, this will return FALSE and
-           * we effectively fail the state change. */
-          if (!gst_element_set_clock (element, clock))
-            goto invalid_clock;
-
-          /* if we selected and distributed a new clock, let the app 
-           * know about it */
-          gst_element_post_message (element,
-              gst_message_new_new_clock (GST_OBJECT_CAST (element), clock));
-        }
-
-        if (stream_time != GST_CLOCK_TIME_NONE)
-          new_base_time = start_time - stream_time + delay;
-        else
-          new_base_time = GST_CLOCK_TIME_NONE;
-
-        gst_object_unref (clock);
       } else {
-        GST_DEBUG ("no clock, using base time of 0");
-        new_base_time = 0;
+        GST_DEBUG ("no clock, using base time of NONE");
+        start_time = GST_CLOCK_TIME_NONE;
+        new_base_time = GST_CLOCK_TIME_NONE;
       }
 
+      GST_OBJECT_LOCK (element);
+      new_clock = element->clock != clock;
+      stream_time = pipeline->stream_time;
+      delay = pipeline->delay;
+      GST_OBJECT_UNLOCK (element);
+
+      if (new_clock) {
+        /* now distribute the clock (which could be NULL). If some
+         * element refuses the clock, this will return FALSE and
+         * we effectively fail the state change. */
+        if (!gst_element_set_clock (element, clock))
+          goto invalid_clock;
+
+        /* if we selected and distributed a new clock, let the app 
+         * know about it */
+        gst_element_post_message (element,
+            gst_message_new_new_clock (GST_OBJECT_CAST (element), clock));
+      }
+
+      if (stream_time != GST_CLOCK_TIME_NONE
+          && start_time != GST_CLOCK_TIME_NONE) {
+        new_base_time = start_time - stream_time + delay;
+        GST_DEBUG_OBJECT (element,
+            "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
+            ", base_time %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (stream_time), GST_TIME_ARGS (start_time),
+            GST_TIME_ARGS (new_base_time));
+      } else
+        new_base_time = GST_CLOCK_TIME_NONE;
+
+      if (clock)
+        gst_object_unref (clock);
+
       if (new_base_time != GST_CLOCK_TIME_NONE)
         gst_element_set_base_time (element, new_base_time);
       else
         GST_DEBUG_OBJECT (pipeline,
             "NOT adjusting base time because stream time is NONE");
-    }
       break;
+    }
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -496,8 +505,8 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
     {
       gboolean need_reset;
 
-      /* only reset the stream time when the application did not
-       * specify a stream time explicitly */
+      /* only reset the stream_time when the application did not
+       * specify a stream_time explicitly */
       GST_OBJECT_LOCK (element);
       need_reset = pipeline->stream_time != GST_CLOCK_TIME_NONE;
       GST_OBJECT_UNLOCK (element);
@@ -527,7 +536,7 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
           pipeline->stream_time = now - element->base_time;
         GST_DEBUG_OBJECT (element,
             "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
-            ", base time %" GST_TIME_FORMAT,
+            ", base_time %" GST_TIME_FORMAT,
             GST_TIME_ARGS (pipeline->stream_time), GST_TIME_ARGS (now),
             GST_TIME_ARGS (element->base_time));
       }
@@ -559,6 +568,8 @@ invalid_clock:
         ("Pipeline cannot operate with selected clock"));
     GST_DEBUG_OBJECT (pipeline,
         "Pipeline cannot operate with selected clock %p", clock);
+    if (clock)
+      gst_object_unref (clock);
     return GST_STATE_CHANGE_FAILURE;
   }
 }
@@ -609,7 +620,7 @@ gst_pipeline_set_new_stream_time (GstPipeline * pipeline, GstClockTime time)
       GST_TIME_ARGS (time));
 
   if (time == GST_CLOCK_TIME_NONE)
-    GST_DEBUG_OBJECT (pipeline, "told not to adjust base time");
+    GST_DEBUG_OBJECT (pipeline, "told not to adjust base_time");
 }
 
 /**
@@ -653,7 +664,8 @@ gst_pipeline_provide_clock_func (GstElement * element)
   GST_OBJECT_LOCK (pipeline);
   if (GST_OBJECT_FLAG_IS_SET (pipeline, GST_PIPELINE_FLAG_FIXED_CLOCK)) {
     clock = pipeline->fixed_clock;
-    gst_object_ref (clock);
+    if (clock)
+      gst_object_ref (clock);
     GST_OBJECT_UNLOCK (pipeline);
 
     GST_CAT_DEBUG (GST_CAT_CLOCK, "pipeline using fixed clock %p (%s)",
@@ -789,7 +801,8 @@ gst_pipeline_auto_clock (GstPipeline * pipeline)
  * Set the expected delay needed for all elements to perform the
  * PAUSED to PLAYING state change. @delay will be added to the
  * base time of the elements so that they wait an additional @delay
- * amount of time before starting to process buffers.
+ * amount of time before starting to process buffers and cannot be
+ * #GST_CLOCK_TIME_NONE.
  *
  * This option is used for tuning purposes and should normally not be 
  * used.
@@ -802,6 +815,7 @@ void
 gst_pipeline_set_delay (GstPipeline * pipeline, GstClockTime delay)
 {
   g_return_if_fail (GST_IS_PIPELINE (pipeline));
+  g_return_if_fail (delay != GST_CLOCK_TIME_NONE);
 
   GST_OBJECT_LOCK (pipeline);
   pipeline->delay = delay;
