@@ -206,6 +206,9 @@ _gst_tag_initialize (void)
       G_TYPE_DOUBLE, _("replaygain album gain"), _("album gain in db"), NULL);
   gst_tag_register (GST_TAG_ALBUM_PEAK, GST_TAG_FLAG_META,
       G_TYPE_DOUBLE, _("replaygain album peak"), _("peak of the album"), NULL);
+  gst_tag_register (GST_TAG_REFERENCE_LEVEL, GST_TAG_FLAG_META,
+      G_TYPE_DOUBLE, _("replaygain reference level"),
+      _("reference level of track and album gain values"), NULL);
   gst_tag_register (GST_TAG_LANGUAGE_CODE, GST_TAG_FLAG_META, G_TYPE_STRING,
       _("language code"),
       _("language code for this stream, conforming to ISO-639-1"), NULL);
@@ -213,6 +216,9 @@ _gst_tag_initialize (void)
       _("image"), _("image related to this stream"), gst_tag_merge_use_first);
   gst_tag_register (GST_TAG_PREVIEW_IMAGE, GST_TAG_FLAG_META, GST_TYPE_BUFFER,
       _("preview image"), _("preview image related to this stream"), NULL);
+  gst_tag_register (GST_TAG_BEATS_PER_MINUTE, GST_TAG_FLAG_META, G_TYPE_DOUBLE,
+      _("beats per minute"), _("number of beats per minute in audio"), NULL);
+
 }
 
 /**
