@@ -58,7 +58,7 @@ gst_uri_handler_get_type (void)
 {
   static GType urihandler_type = 0;
 
-  if (!urihandler_type) {
+  if (G_UNLIKELY (urihandler_type == 0)) {
     static const GTypeInfo urihandler_info = {
       sizeof (GstURIHandlerInterface),
       gst_uri_handler_base_init,
@@ -372,6 +372,34 @@ gst_uri_get_protocol (const gchar * uri)
   return g_strndup (uri, colon - uri);
 }
 
+/**
+ * gst_uri_has_protocol:
+ * @uri: an URI string
+ * @protocol: a protocol string (e.g. "http")
+ *
+ * Checks if the protocol of a given valid URI matches @protocol.
+ *
+ * Returns: %TRUE if the protocol matches.
+ *
+ * Since: 0.10.4
+ */
+gboolean
+gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
+{
+  gchar *colon;
+
+  g_return_val_if_fail (uri != NULL, FALSE);
+  g_return_val_if_fail (protocol != NULL, FALSE);
+  g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
+
+  colon = strstr (uri, "://");
+
+  if (colon == NULL)
+    return FALSE;
+
+  return (strncmp (uri, protocol, (size_t) (colon - uri)) == 0);
+}
+
 /**
  * gst_uri_get_location:
  * @uri: A URI string
@@ -449,8 +477,13 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
     return FALSE;
 
   protocols = gst_element_factory_get_uri_protocols (factory);
-  /* must be set when uri type is valid */
-  g_assert (protocols);
+
+  if (protocols == NULL) {
+    g_warning ("Factory '%s' implements GstUriHandler interface but returned "
+        "no supported protocols!", gst_plugin_feature_get_name (feature));
+    return FALSE;
+  }
+
   while (*protocols != NULL) {
     if (strcmp (*protocols, entry->protocol) == 0)
       return TRUE;
@@ -583,8 +616,9 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
  *
  * Gets the currently handled URI.
  *
- * Returns: the URI currently handler by the @handler.
- * Returns NULL if there are no URI currently handled.
+ * Returns: the URI currently handled by the @handler.
+ * Returns NULL if there are no URI currently handled. The returned
+ * string must not be modified or freed.
  */
 G_CONST_RETURN gchar *
 gst_uri_handler_get_uri (GstURIHandler * handler)
