@@ -125,8 +125,10 @@
  *
  * The qos property will enable the quality-of-service features of the basesink
  * which gather statistics about the real-time performance of the clock
- * synchronisation. For each dropped buffer it will also send a QoS message
- * upstream.
+ * synchronisation. For each buffer received in the sink, statistics are
+ * gathered and a QOS event is send upstream with these numbers. This
+ * information can then be used by upstream elements to reduce their processing
+ * rate, for example.
  *
  * Last reviewed on 2006-09-27 (0.10.11)
  */
@@ -187,6 +189,9 @@ struct _GstBaseSinkPrivate
   /* number of rendered and dropped frames */
   guint64 rendered;
   guint64 dropped;
+
+  /* latency stuff */
+  GstClockTime latency;
 };
 
 #define DO_RUNNING_AVG(avg,val,size) (((val) + ((size)-1) * (avg)) / (size))
@@ -274,16 +279,19 @@ static void gst_base_sink_get_times (GstBaseSink * basesink, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end);
 static gboolean gst_base_sink_set_flushing (GstBaseSink * basesink,
     GstPad * pad, gboolean flushing);
+static gboolean gst_base_sink_default_activate_pull (GstBaseSink * basesink,
+    gboolean active);
 
 static GstStateChangeReturn gst_base_sink_change_state (GstElement * element,
     GstStateChange transition);
 
 static GstFlowReturn gst_base_sink_chain (GstPad * pad, GstBuffer * buffer);
 static void gst_base_sink_loop (GstPad * pad);
-static gboolean gst_base_sink_activate (GstPad * pad);
-static gboolean gst_base_sink_activate_push (GstPad * pad, gboolean active);
-static gboolean gst_base_sink_activate_pull (GstPad * pad, gboolean active);
+static gboolean gst_base_sink_pad_activate (GstPad * pad);
+static gboolean gst_base_sink_pad_activate_push (GstPad * pad, gboolean active);
+static gboolean gst_base_sink_pad_activate_pull (GstPad * pad, gboolean active);
 static gboolean gst_base_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_base_sink_peer_query (GstBaseSink * sink, GstQuery * query);
 
 /* check if an object was too late */
 static gboolean gst_base_sink_is_too_late (GstBaseSink * basesink,
@@ -332,8 +340,9 @@ gst_base_sink_class_init (GstBaseSinkClass * klass)
           G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_QOS,
-      g_param_spec_boolean ("qos", "Qos", "Generate QoS events upstream",
-          DEFAULT_QOS, G_PARAM_READWRITE));
+      g_param_spec_boolean ("qos", "Qos",
+          "Generate Quality-of-Service events upstream", DEFAULT_QOS,
+          G_PARAM_READWRITE));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_sink_change_state);
@@ -344,6 +353,8 @@ gst_base_sink_class_init (GstBaseSinkClass * klass)
   klass->set_caps = GST_DEBUG_FUNCPTR (gst_base_sink_set_caps);
   klass->buffer_alloc = GST_DEBUG_FUNCPTR (gst_base_sink_buffer_alloc);
   klass->get_times = GST_DEBUG_FUNCPTR (gst_base_sink_get_times);
+  klass->activate_pull =
+      GST_DEBUG_FUNCPTR (gst_base_sink_default_activate_pull);
 }
 
 static GstCaps *
@@ -377,12 +388,24 @@ gst_base_sink_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstBaseSinkClass *bclass;
   GstBaseSink *bsink;
-  gboolean res = FALSE;
+  gboolean res = TRUE;
 
   bsink = GST_BASE_SINK (gst_pad_get_parent (pad));
   bclass = GST_BASE_SINK_GET_CLASS (bsink);
 
-  if (bclass->set_caps)
+  if (bsink->pad_mode == GST_ACTIVATE_PULL) {
+    GstPad *peer = gst_pad_get_peer (pad);
+
+    if (peer)
+      res = gst_pad_set_caps (peer, caps);
+    else
+      res = FALSE;
+
+    if (!res)
+      GST_DEBUG_OBJECT (bsink, "peer setcaps() failed");
+  }
+
+  if (res && bclass->set_caps)
     res = bclass->set_caps (bsink, caps);
 
   gst_object_unref (bsink);
@@ -390,6 +413,21 @@ gst_base_sink_pad_setcaps (GstPad * pad, GstCaps * caps)
   return res;
 }
 
+static void
+gst_base_sink_pad_fixate (GstPad * pad, GstCaps * caps)
+{
+  GstBaseSinkClass *bclass;
+  GstBaseSink *bsink;
+
+  bsink = GST_BASE_SINK (gst_pad_get_parent (pad));
+  bclass = GST_BASE_SINK_GET_CLASS (bsink);
+
+  if (bclass->fixate)
+    bclass->fixate (bsink, caps);
+
+  gst_object_unref (bsink);
+}
+
 static GstFlowReturn
 gst_base_sink_pad_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     GstCaps * caps, GstBuffer ** buf)
@@ -429,14 +467,16 @@ gst_base_sink_init (GstBaseSink * basesink, gpointer g_class)
       GST_DEBUG_FUNCPTR (gst_base_sink_pad_getcaps));
   gst_pad_set_setcaps_function (basesink->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_sink_pad_setcaps));
+  gst_pad_set_fixatecaps_function (basesink->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_base_sink_pad_fixate));
   gst_pad_set_bufferalloc_function (basesink->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_sink_pad_buffer_alloc));
   gst_pad_set_activate_function (basesink->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_base_sink_activate));
+      GST_DEBUG_FUNCPTR (gst_base_sink_pad_activate));
   gst_pad_set_activatepush_function (basesink->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_base_sink_activate_push));
+      GST_DEBUG_FUNCPTR (gst_base_sink_pad_activate_push));
   gst_pad_set_activatepull_function (basesink->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_base_sink_activate_pull));
+      GST_DEBUG_FUNCPTR (gst_base_sink_pad_activate_pull));
   gst_pad_set_event_function (basesink->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_sink_event));
   gst_pad_set_chain_function (basesink->sinkpad,
@@ -564,7 +604,7 @@ gst_base_sink_get_max_lateness (GstBaseSink * sink)
  * @sink: the sink
  * @enabled: the new qos value.
  *
- * Configures @sink to send QoS events upstream.
+ * Configures @sink to send Quality-of-Service events upstream.
  *
  * Since: 0.10.5
  */
@@ -578,10 +618,10 @@ gst_base_sink_set_qos_enabled (GstBaseSink * sink, gboolean enabled)
  * gst_base_sink_is_qos_enabled:
  * @sink: the sink
  *
- * Checks if @sink is currently configured to send QoS events
+ * Checks if @sink is currently configured to send Quality-of-Service events
  * upstream.
  *
- * Returns: TRUE if the sink is configured to perform QoS.
+ * Returns: TRUE if the sink is configured to perform Quality-of-Service.
  *
  * Since: 0.10.5
  */
@@ -595,6 +635,110 @@ gst_base_sink_is_qos_enabled (GstBaseSink * sink)
   return res;
 }
 
+/**
+ * gst_base_sink_get_latency:
+ * @sink: the sink
+ *
+ * Get the currently configured latency.
+ *
+ * Returns: The configured latency.
+ *
+ * Since: 0.10.12
+ */
+GstClockTime
+gst_base_sink_get_latency (GstBaseSink * sink)
+{
+  GstClockTime res;
+
+  GST_OBJECT_LOCK (sink);
+  res = sink->priv->latency;
+  GST_OBJECT_UNLOCK (sink);
+
+  return res;
+}
+
+/**
+ * gst_base_sink_query_latency:
+ * @sink: the sink
+ * @live: if the sink is live
+ * @upstream_live: if an upstream element is live
+ * @min_latency: the min latency of the upstream elements
+ * @max_latency: the max latency of the upstream elements
+ *
+ * Query the sink for the latency parameters. The latency will be queried from
+ * the upstream elements. @live will be TRUE if @sink is configured to
+ * synchronize against the clock. @upstream_live will be TRUE if an upstream
+ * element is live. 
+ *
+ * If both @live and @upstream_live are TRUE, the sink will want to compensate
+ * for the latency introduced by the upstream elements by setting the
+ * @min_latency to a strictly possitive value.
+ *
+ * This function is mostly used by subclasses. 
+ *
+ * Returns: TRUE if the query succeeded.
+ *
+ * Since: 0.10.12
+ */
+gboolean
+gst_base_sink_query_latency (GstBaseSink * sink, gboolean * live,
+    gboolean * upstream_live, GstClockTime * min_latency,
+    GstClockTime * max_latency)
+{
+  gboolean l, us_live, res;
+  GstClockTime min, max;
+  GstQuery *query;
+  GstClockTime us_min, us_max;
+
+  /* we are live when we sync to the clock */
+  l = gst_base_sink_get_sync (sink);
+
+  /* assume no latency */
+  min = 0;
+  max = -1;
+  us_live = FALSE;
+
+  query = gst_query_new_latency ();
+
+  /* ask the peer for the latency */
+  if (!(res = gst_base_sink_peer_query (sink, query)))
+    goto query_failed;
+
+  /* get upstream min and max latency */
+  gst_query_parse_latency (query, &us_live, &us_min, &us_max);
+  if (us_live) {
+    /* upstream live, use its latency, subclasses should use these
+     * values to create the complete latency. */
+    min = us_min;
+    max = us_max;
+  }
+
+  GST_DEBUG_OBJECT (sink, "latency query: live: %d, upstream: %d, min %"
+      GST_TIME_FORMAT ", max %" GST_TIME_FORMAT, l, us_live,
+      GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+  if (live)
+    *live = l;
+  if (upstream_live)
+    *upstream_live = us_live;
+  if (min_latency)
+    *min_latency = min;
+  if (max_latency)
+    *max_latency = max;
+
+done:
+  gst_query_unref (query);
+
+  return res;
+
+  /* ERRORS */
+query_failed:
+  {
+    GST_DEBUG_OBJECT (sink, "latency query failed");
+    goto done;
+  }
+}
+
 static void
 gst_base_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -1008,7 +1152,6 @@ gst_base_sink_wait_clock (GstBaseSink * basesink, GstClockTime time,
   GstClockID id;
   GstClockReturn ret;
   GstClock *clock;
-  GstClockTime base_time;
 
   if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (time)))
     goto invalid_time;
@@ -1020,8 +1163,11 @@ gst_base_sink_wait_clock (GstBaseSink * basesink, GstClockTime time,
   if (G_UNLIKELY ((clock = GST_ELEMENT_CLOCK (basesink)) == NULL))
     goto no_clock;
 
-  base_time = GST_ELEMENT_CAST (basesink)->base_time;
-  id = gst_clock_new_single_shot_id (clock, base_time + time);
+  /* add base time and latency */
+  time += GST_ELEMENT_CAST (basesink)->base_time;
+  time += basesink->priv->latency;
+
+  id = gst_clock_new_single_shot_id (clock, time);
   GST_OBJECT_UNLOCK (basesink);
 
   basesink->clock_id = id;
@@ -1242,7 +1388,7 @@ gst_base_sink_send_qos (GstBaseSink * basesink,
   GstEvent *event;
   gboolean res;
 
-  /* generate QoS event */
+  /* generate Quality-of-Service event */
   GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, basesink,
       "qos: proportion: %lf, diff %" G_GINT64_FORMAT ", timestamp %"
       GST_TIME_FORMAT, proportion, diff, GST_TIME_ARGS (time));
@@ -1269,7 +1415,7 @@ gst_base_sink_perform_qos (GstBaseSink * sink, gboolean dropped)
 
   start = priv->current_rstart;
 
-  /* if QoS disabled, do nothing */
+  /* if Quality-of-Service disabled, do nothing */
   if (!g_atomic_int_get (&priv->qos_enabled) || start == -1)
     return;
 
@@ -1704,8 +1850,10 @@ gst_base_sink_queue_object_unlocked (GstBaseSink * basesink, GstPad * pad,
     o = g_queue_pop_head (q);
     GST_DEBUG_OBJECT (basesink, "rendering queued object %p", o);
 
-    /* FIXME, do something with the return value? */
+    /* do something with the return value */
     ret = gst_base_sink_render_object (basesink, pad, o);
+    if (ret != GST_FLOW_OK)
+      goto dequeue_failed;
   }
 
   /* now render the object */
@@ -1730,6 +1878,13 @@ more_preroll:
     g_queue_push_tail (basesink->preroll_queue, obj);
     return GST_FLOW_OK;
   }
+dequeue_failed:
+  {
+    GST_DEBUG_OBJECT (basesink, "rendering queued objects failed, reason %s",
+        gst_flow_get_name (ret));
+    gst_mini_object_unref (obj);
+    return ret;
+  }
 }
 
 /* with STREAM_LOCK
@@ -1796,6 +1951,8 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
     {
       GstFlowReturn ret;
 
+      GST_DEBUG_OBJECT (basesink, "newsegment %p", event);
+
       basesink->have_newsegment = TRUE;
 
       /* the new segment is a non prerollable item and does not block anything,
@@ -1815,6 +1972,8 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
       if (bclass->event)
         bclass->event (basesink, event);
 
+      GST_DEBUG_OBJECT (basesink, "flush-start %p", event);
+
       /* make sure we are not blocked on the clock also clear any pending
        * eos state. */
       gst_base_sink_set_flushing (basesink, pad, TRUE);
@@ -1828,7 +1987,6 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
        * prerolled buffer */
       basesink->playing_async = TRUE;
       gst_element_lost_state (GST_ELEMENT_CAST (basesink));
-      GST_DEBUG_OBJECT (basesink, "event unref %p %p", basesink, event);
       GST_PAD_STREAM_UNLOCK (pad);
 
       gst_event_unref (event);
@@ -1837,6 +1995,8 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
       if (bclass->event)
         bclass->event (basesink, event);
 
+      GST_DEBUG_OBJECT (basesink, "flush-stop %p", event);
+
       /* unset flushing so we can accept new data */
       gst_base_sink_set_flushing (basesink, pad, FALSE);
 
@@ -1846,7 +2006,6 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
           GST_FORMAT_UNDEFINED);
       basesink->have_newsegment = FALSE;
 
-      GST_DEBUG_OBJECT (basesink, "event unref %p %p", basesink, event);
       gst_event_unref (event);
       break;
     default:
@@ -1925,10 +2084,7 @@ gst_base_sink_chain_unlocked (GstBaseSink * basesink, GstPad * pad,
   if (G_UNLIKELY (!basesink->have_newsegment)) {
     gboolean sync;
 
-    GST_OBJECT_LOCK (basesink);
-    sync = basesink->sync;
-    GST_OBJECT_UNLOCK (basesink);
-
+    sync = gst_base_sink_get_sync (basesink);
     if (sync) {
       GST_ELEMENT_WARNING (basesink, STREAM, FAILED,
           (_("Internal data flow problem.")),
@@ -2114,20 +2270,24 @@ gst_base_sink_set_flushing (GstBaseSink * basesink, GstPad * pad,
 }
 
 static gboolean
-gst_base_sink_deactivate (GstBaseSink * basesink, GstPad * pad)
+gst_base_sink_default_activate_pull (GstBaseSink * basesink, gboolean active)
 {
   gboolean result;
 
-  gst_base_sink_set_flushing (basesink, pad, TRUE);
-
-  /* step 2, make sure streaming finishes */
-  result = gst_pad_stop_task (pad);
+  if (active) {
+    /* start task */
+    result = gst_pad_start_task (basesink->sinkpad,
+        (GstTaskFunction) gst_base_sink_loop, basesink->sinkpad);
+  } else {
+    /* step 2, make sure streaming finishes */
+    result = gst_pad_stop_task (basesink->sinkpad);
+  }
 
   return result;
 }
 
 static gboolean
-gst_base_sink_activate (GstPad * pad)
+gst_base_sink_pad_activate (GstPad * pad)
 {
   gboolean result = FALSE;
   GstBaseSink *basesink;
@@ -2161,7 +2321,7 @@ gst_base_sink_activate (GstPad * pad)
 }
 
 static gboolean
-gst_base_sink_activate_push (GstPad * pad, gboolean active)
+gst_base_sink_pad_activate_push (GstPad * pad, gboolean active)
 {
   gboolean result;
   GstBaseSink *basesink;
@@ -2181,7 +2341,8 @@ gst_base_sink_activate_push (GstPad * pad, gboolean active)
       g_warning ("Internal GStreamer activation error!!!");
       result = FALSE;
     } else {
-      result = gst_base_sink_deactivate (basesink, pad);
+      gst_base_sink_set_flushing (basesink, pad, TRUE);
+      result = TRUE;
       basesink->pad_mode = GST_ACTIVATE_NONE;
     }
   }
@@ -2191,14 +2352,66 @@ gst_base_sink_activate_push (GstPad * pad, gboolean active)
   return result;
 }
 
+static gboolean
+gst_base_sink_negotiate_pull (GstBaseSink * basesink)
+{
+  GstCaps *caps;
+  GstPad *pad;
+
+  GST_OBJECT_LOCK (basesink);
+  pad = basesink->sinkpad;
+  gst_object_ref (pad);
+  GST_OBJECT_UNLOCK (basesink);
+
+  caps = gst_pad_get_allowed_caps (pad);
+  if (gst_caps_is_empty (caps))
+    goto no_caps_possible;
+
+  caps = gst_caps_make_writable (caps);
+  gst_caps_truncate (caps);
+  gst_pad_fixate_caps (pad, caps);
+
+  if (gst_caps_is_any (caps)) {
+    GST_DEBUG_OBJECT (basesink, "caps were ANY after fixating, "
+        "allowing pull()");
+    /* neither side has template caps in this case, so they are prepared for
+       pull() without setcaps() */
+  } else {
+    if (!gst_pad_set_caps (pad, caps))
+      goto could_not_set_caps;
+  }
+
+  gst_caps_unref (caps);
+  gst_object_unref (pad);
+
+  return TRUE;
+
+no_caps_possible:
+  {
+    GST_INFO_OBJECT (basesink, "Pipeline could not agree on caps");
+    GST_DEBUG_OBJECT (basesink, "get_allowed_caps() returned EMPTY");
+    gst_object_unref (pad);
+    return FALSE;
+  }
+could_not_set_caps:
+  {
+    GST_INFO_OBJECT (basesink, "Could not set caps: %" GST_PTR_FORMAT, caps);
+    gst_caps_unref (caps);
+    gst_object_unref (pad);
+    return FALSE;
+  }
+}
+
 /* this won't get called until we implement an activate function */
 static gboolean
-gst_base_sink_activate_pull (GstPad * pad, gboolean active)
+gst_base_sink_pad_activate_pull (GstPad * pad, gboolean active)
 {
   gboolean result = FALSE;
   GstBaseSink *basesink;
+  GstBaseSinkClass *bclass;
 
   basesink = GST_BASE_SINK (gst_pad_get_parent (pad));
+  bclass = GST_BASE_SINK_GET_CLASS (basesink);
 
   if (active) {
     if (!basesink->can_activate_pull) {
@@ -2222,11 +2435,15 @@ gst_base_sink_activate_pull (GstPad * pad, gboolean active)
           basesink->have_newsegment = TRUE;
 
           /* set the pad mode before starting the task so that it's in the
-             correct state for the new thread... */
+             correct state for the new thread. also the sink set_caps function
+             checks this */
           basesink->pad_mode = GST_ACTIVATE_PULL;
-          result =
-              gst_pad_start_task (pad, (GstTaskFunction) gst_base_sink_loop,
-              pad);
+          if ((result = gst_base_sink_negotiate_pull (basesink))) {
+            if (bclass->activate_pull)
+              result = bclass->activate_pull (basesink, TRUE);
+            else
+              result = FALSE;
+          }
           /* but if starting the thread fails, set it back */
           if (!result)
             basesink->pad_mode = GST_ACTIVATE_NONE;
@@ -2243,7 +2460,9 @@ gst_base_sink_activate_pull (GstPad * pad, gboolean active)
       g_warning ("Internal GStreamer activation error!!!");
       result = FALSE;
     } else {
-      result = gst_base_sink_deactivate (basesink, pad);
+      result = gst_base_sink_set_flushing (basesink, pad, TRUE);
+      if (bclass->activate_pull)
+        result &= bclass->activate_pull (basesink, FALSE);
       basesink->pad_mode = GST_ACTIVATE_NONE;
     }
   }
@@ -2259,17 +2478,39 @@ gst_base_sink_send_event (GstElement * element, GstEvent * event)
 {
   GstPad *pad;
   GstBaseSink *basesink = GST_BASE_SINK (element);
-  gboolean result;
+  gboolean forward = TRUE, result = TRUE;
 
-  GST_OBJECT_LOCK (element);
-  pad = basesink->sinkpad;
-  gst_object_ref (pad);
-  GST_OBJECT_UNLOCK (element);
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_LATENCY:
+    {
+      GstClockTime latency;
 
-  result = gst_pad_push_event (pad, event);
+      gst_event_parse_latency (event, &latency);
 
-  gst_object_unref (pad);
+      GST_OBJECT_LOCK (element);
+      basesink->priv->latency = latency;
+      GST_OBJECT_UNLOCK (element);
+      GST_DEBUG_OBJECT (basesink, "latency set to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (latency));
 
+      /* don't forward, yet */
+      forward = FALSE;
+      gst_event_unref (event);
+      break;
+    }
+    default:
+      break;
+  }
+
+  if (forward) {
+    GST_OBJECT_LOCK (element);
+    pad = gst_object_ref (basesink->sinkpad);
+    GST_OBJECT_UNLOCK (element);
+
+    result = gst_pad_push_event (pad, event);
+
+    gst_object_unref (pad);
+  }
   return result;
 }
 
@@ -2479,7 +2720,25 @@ gst_base_sink_query (GstElement * element, GstQuery * query)
       res = gst_base_sink_peer_query (basesink, query);
       break;
     case GST_QUERY_LATENCY:
+    {
+      gboolean live, us_live;
+      GstClockTime min, max;
+
+      if ((res =
+              gst_base_sink_query_latency (basesink, &live, &us_live, &min,
+                  &max))) {
+        /* if we or the upstream elements are not live, we don't need latency
+         * compensation */
+        if (!live || !us_live) {
+          GST_DEBUG_OBJECT (basesink,
+              "no latency compensation, we or upstream are not live");
+          min = 0;
+          max = -1;
+        }
+        gst_query_set_latency (query, live, min, max);
+      }
       break;
+    }
     case GST_QUERY_JITTER:
       break;
     case GST_QUERY_RATE:
@@ -2533,6 +2792,7 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
       basesink->priv->current_sstart = 0;
       basesink->priv->current_sstop = 0;
       basesink->priv->eos_rtime = -1;
+      basesink->priv->latency = 0;
       basesink->eos = FALSE;
       gst_base_sink_reset_qos (basesink);
       ret = GST_STATE_CHANGE_ASYNC;
@@ -2576,6 +2836,7 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED");
+      /* FIXME, make sure we cannot enter _render first */
 
       /* we need to call ::unlock before locking PREROLL_LOCK
        * since we lock it before going into ::render */
@@ -2591,6 +2852,7 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
       /* if we don't have a preroll buffer we need to wait for a preroll and
        * return ASYNC. */
       if (gst_base_sink_is_prerolled (basesink)) {
+        GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED, we are prerolled");
         basesink->playing_async = FALSE;
       } else {
         GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED, need preroll");
