@@ -269,6 +269,7 @@ gst_base_src_get_type (void)
 }
 static GstCaps *gst_base_src_getcaps (GstPad * pad);
 static gboolean gst_base_src_setcaps (GstPad * pad, GstCaps * caps);
+static void gst_base_src_fixate (GstPad * pad, GstCaps * caps);
 
 static gboolean gst_base_src_activate_push (GstPad * pad, gboolean active);
 static gboolean gst_base_src_activate_pull (GstPad * pad, gboolean active);
@@ -391,6 +392,8 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
       GST_DEBUG_FUNCPTR (gst_base_src_pad_get_range));
   gst_pad_set_getcaps_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_getcaps));
   gst_pad_set_setcaps_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_setcaps));
+  gst_pad_set_fixatecaps_function (pad,
+      GST_DEBUG_FUNCPTR (gst_base_src_fixate));
 
   /* hold pointer to pad */
   basesrc->srcpad = pad;
@@ -578,6 +581,21 @@ gst_base_src_getcaps (GstPad * pad)
   return caps;
 }
 
+static void
+gst_base_src_fixate (GstPad * pad, GstCaps * caps)
+{
+  GstBaseSrcClass *bclass;
+  GstBaseSrc *bsrc;
+
+  bsrc = GST_BASE_SRC (gst_pad_get_parent (pad));
+  bclass = GST_BASE_SRC_GET_CLASS (bsrc);
+
+  if (bclass->fixate)
+    bclass->fixate (bsrc, caps);
+
+  gst_object_unref (bsrc);
+}
+
 static gboolean
 gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
 {
@@ -717,6 +735,14 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       break;
     }
     case GST_QUERY_LATENCY:
+      /* we can only report the fact that we are live or not, we know nothing
+       * about latency. Subclasses should override and implement something
+       * usefull */
+      GST_LIVE_LOCK (src);
+      gst_query_set_latency (query, src->is_live, 0, -1);
+      GST_LIVE_UNLOCK (src);
+      res = TRUE;
+      break;
     case GST_QUERY_JITTER:
     case GST_QUERY_RATE:
     default:
@@ -1521,7 +1547,8 @@ gst_base_src_loop (GstPad * pad)
 
   ret = gst_base_src_get_range (src, position, src->blocksize, &buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
-    GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() = %d", ret);
+    GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() = %s",
+        gst_flow_get_name (ret));
     goto pause;
   }
   if (G_UNLIKELY (buf == NULL))
@@ -1573,7 +1600,8 @@ gst_base_src_loop (GstPad * pad)
 
   ret = gst_pad_push (pad, buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
-    GST_INFO_OBJECT (src, "pausing after gst_pad_push() = %d", ret);
+    GST_INFO_OBJECT (src, "pausing after gst_pad_push() = %s",
+        gst_flow_get_name (ret));
     goto pause;
   }
 
@@ -1930,12 +1958,12 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
   /* ERRORS */
 no_push_activation:
   {
-    GST_ERROR_OBJECT (basesrc, "Subclass disabled push-mode activation");
+    GST_WARNING_OBJECT (basesrc, "Subclass disabled push-mode activation");
     return FALSE;
   }
 error_start:
   {
-    GST_ERROR_OBJECT (basesrc, "Failed to start in push mode");
+    GST_WARNING_OBJECT (basesrc, "Failed to start in push mode");
     return FALSE;
   }
 seek_failed:
