@@ -121,7 +121,7 @@ struct _GstBaseSrc {
  * @get_caps: Called to get the caps to report
  * @set_caps: Notify subclass of changed output caps
  * @negotiate: Negotiated the caps with the peer.
- * @newsegment: Generate and send a new_segment event.
+ * @newsegment: Generate and send a new_segment event (UNUSED)
  * @start: Start processing. Subclasses should open resources and prepare
  *    to produce data.
  * @stop: Stop processing. Subclasses should use this to close resources.
@@ -142,6 +142,12 @@ struct _GstBaseSrc {
  *   cycles. The default implementation will open and close the resource 
  *   to find out whether get_range is supported, and that is usually 
  *   undesirable. 
+ * @fixate: Called during negotation if caps need fixating. Implement instead of
+ *   setting a fixate function on the source pad.
+ *
+ * Subclasses can override any of the available virtual methods or not, as
+ * needed. At the minimum, the @create method should be overridden to produce
+ * buffers.
  */
 struct _GstBaseSrcClass {
   GstElementClass parent_class;
@@ -157,7 +163,7 @@ struct _GstBaseSrcClass {
   /* decide on caps */
   gboolean      (*negotiate)    (GstBaseSrc *src);
 
-  /* generate and send a newsegment */
+  /* generate and send a newsegment (UNUSED) */
   gboolean      (*newsegment)   (GstBaseSrc *src);
 
   /* start and stop processing, ideal for opening/closing the resource */
@@ -199,16 +205,21 @@ struct _GstBaseSrcClass {
    * undesirable. */
   gboolean      (*check_get_range) (GstBaseSrc *src);
 
+  /* called if, in negotation, caps need fixating */
+  void		(*fixate)	(GstBaseSrc *src, GstCaps *caps);
+
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE - 3];
+  gpointer       _gst_reserved[GST_PADDING_LARGE - 4];
 };
 
 GType gst_base_src_get_type (void);
 
-void		gst_base_src_set_live	(GstBaseSrc *src, gboolean live);
-gboolean	gst_base_src_is_live	(GstBaseSrc *src);
+GstFlowReturn   gst_base_src_wait_playing  (GstBaseSrc *src);
 
-void		gst_base_src_set_format	(GstBaseSrc *src, GstFormat format);
+void		gst_base_src_set_live	   (GstBaseSrc *src, gboolean live);
+gboolean	gst_base_src_is_live	   (GstBaseSrc *src);
+
+void		gst_base_src_set_format	   (GstBaseSrc *src, GstFormat format);
 
 G_END_DECLS
 
