@@ -230,6 +230,8 @@ struct _GstBaseTransformPrivate
   gboolean qos_enabled;
   gdouble proportion;
   GstClockTime earliest_time;
+
+  GstActivateMode pad_mode;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -274,18 +276,18 @@ static gboolean gst_base_transform_src_activate_pull (GstPad * pad,
     gboolean active);
 static gboolean gst_base_transform_sink_activate_push (GstPad * pad,
     gboolean active);
+static gboolean gst_base_transform_activate (GstBaseTransform * trans,
+    gboolean active);
 static gboolean gst_base_transform_get_unit_size (GstBaseTransform * trans,
     GstCaps * caps, guint * size);
 
-static GstStateChangeReturn gst_base_transform_change_state (GstElement *
-    element, GstStateChange transition);
-
 static gboolean gst_base_transform_src_event (GstPad * pad, GstEvent * event);
 static gboolean gst_base_transform_src_eventfunc (GstBaseTransform * trans,
     GstEvent * event);
 static gboolean gst_base_transform_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_base_transform_sink_eventfunc (GstBaseTransform * trans,
     GstEvent * event);
+static gboolean gst_base_transform_check_get_range (GstPad * pad);
 static GstFlowReturn gst_base_transform_getrange (GstPad * pad, guint64 offset,
     guint length, GstBuffer ** buffer);
 static GstFlowReturn gst_base_transform_chain (GstPad * pad,
@@ -320,10 +322,8 @@ static void
 gst_base_transform_class_init (GstBaseTransformClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gstelement_class = GST_ELEMENT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GstBaseTransformPrivate));
 
@@ -340,9 +340,6 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_transform_finalize);
 
-  gstelement_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_base_transform_change_state);
-
   klass->passthrough_on_same_caps = FALSE;
   klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_sink_eventfunc);
   klass->src_event = GST_DEBUG_FUNCPTR (gst_base_transform_src_eventfunc);
@@ -386,6 +383,8 @@ gst_base_transform_init (GstBaseTransform * trans,
       GST_DEBUG_FUNCPTR (gst_base_transform_setcaps));
   gst_pad_set_event_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_src_event));
+  gst_pad_set_checkgetrange_function (trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_check_get_range));
   gst_pad_set_getrange_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_getrange));
   gst_pad_set_activatepull_function (trans->srcpad,
@@ -398,6 +397,7 @@ gst_base_transform_init (GstBaseTransform * trans,
   trans->priv->qos_enabled = DEFAULT_PROP_QOS;
   trans->cache_caps1 = NULL;
   trans->cache_caps2 = NULL;
+  trans->priv->pad_mode = GST_ACTIVATE_NONE;
 
   trans->passthrough = FALSE;
   if (bclass->transform == NULL) {
@@ -811,6 +811,14 @@ gst_base_transform_setcaps (GstPad * pad, GstCaps * caps)
   /* we know this will work, we implement the setcaps */
   gst_pad_set_caps (otherpad, othercaps);
 
+  if (pad == trans->srcpad && trans->priv->pad_mode == GST_ACTIVATE_PULL) {
+    ret &= gst_pad_set_caps (otherpeer, othercaps);
+    if (!ret) {
+      GST_INFO_OBJECT (trans, "otherpeer setcaps(%" GST_PTR_FORMAT ") failed",
+          othercaps);
+    }
+  }
+
 done:
   if (otherpeer)
     gst_object_unref (otherpeer);
@@ -1503,6 +1511,21 @@ configure_failed:
   }
 }
 
+static gboolean
+gst_base_transform_check_get_range (GstPad * pad)
+{
+  GstBaseTransform *trans;
+  gboolean ret;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+
+  ret = gst_pad_check_pull_range (trans->sinkpad);
+
+  gst_object_unref (trans);
+
+  return ret;
+}
+
 /* FIXME, getrange is broken, need to pull range from the other
  * end based on the transform_size result.
  */
@@ -1591,20 +1614,65 @@ gst_base_transform_get_property (GObject * object, guint prop_id,
   }
 }
 
+/* not a vmethod of anything, just an internal method */
+static gboolean
+gst_base_transform_activate (GstBaseTransform * trans, gboolean active)
+{
+  GstBaseTransformClass *bclass;
+  gboolean result = TRUE;
+
+  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
+
+  if (active) {
+    if (trans->priv->pad_mode == GST_ACTIVATE_NONE && bclass->start)
+      result &= bclass->start (trans);
+
+    GST_OBJECT_LOCK (trans);
+
+    if (GST_PAD_CAPS (trans->sinkpad) && GST_PAD_CAPS (trans->srcpad))
+      trans->have_same_caps =
+          gst_caps_is_equal (GST_PAD_CAPS (trans->sinkpad),
+          GST_PAD_CAPS (trans->srcpad)) || trans->passthrough;
+    else
+      trans->have_same_caps = trans->passthrough;
+    GST_DEBUG_OBJECT (trans, "have_same_caps %d", trans->have_same_caps);
+    trans->negotiated = FALSE;
+    trans->have_newsegment = FALSE;
+    gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
+    trans->priv->proportion = 1.0;
+    trans->priv->earliest_time = -1;
+
+    GST_OBJECT_UNLOCK (trans);
+  } else {
+    trans->have_same_caps = FALSE;
+    /* We can only reset the passthrough mode if the instance told us to 
+       handle it in configure_caps */
+    if (bclass->passthrough_on_same_caps) {
+      gst_base_transform_set_passthrough (trans, FALSE);
+    }
+    gst_caps_replace (&trans->cache_caps1, NULL);
+    gst_caps_replace (&trans->cache_caps2, NULL);
+
+    if (trans->priv->pad_mode != GST_ACTIVATE_NONE && bclass->stop)
+      result &= bclass->stop (trans);
+  }
+
+  return result;
+}
+
 static gboolean
 gst_base_transform_sink_activate_push (GstPad * pad, gboolean active)
 {
   gboolean result = TRUE;
   GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
-  if (active) {
-    if (bclass->start)
-      result = bclass->start (trans);
-  }
+  result = gst_base_transform_activate (trans, active);
+
+  if (result)
+    trans->priv->pad_mode = active ? GST_ACTIVATE_PUSH : GST_ACTIVATE_NONE;
+
   gst_object_unref (trans);
 
   return result;
@@ -1615,85 +1683,22 @@ gst_base_transform_src_activate_pull (GstPad * pad, gboolean active)
 {
   gboolean result = FALSE;
   GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
   result = gst_pad_activate_pull (trans->sinkpad, active);
 
-  if (active) {
-    if (result && bclass->start)
-      result &= bclass->start (trans);
-  }
+  if (result)
+    result &= gst_base_transform_activate (trans, active);
+
+  if (result)
+    trans->priv->pad_mode = active ? GST_ACTIVATE_PULL : GST_ACTIVATE_NONE;
+
   gst_object_unref (trans);
 
   return result;
 }
 
-static GstStateChangeReturn
-gst_base_transform_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
-  GstStateChangeReturn result;
-
-  trans = GST_BASE_TRANSFORM (element);
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
-
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      GST_OBJECT_LOCK (trans);
-      if (GST_PAD_CAPS (trans->sinkpad) && GST_PAD_CAPS (trans->srcpad))
-        trans->have_same_caps =
-            gst_caps_is_equal (GST_PAD_CAPS (trans->sinkpad),
-            GST_PAD_CAPS (trans->srcpad)) || trans->passthrough;
-      else
-        trans->have_same_caps = trans->passthrough;
-      GST_DEBUG_OBJECT (trans, "have_same_caps %d", trans->have_same_caps);
-      trans->negotiated = FALSE;
-      trans->have_newsegment = FALSE;
-      gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
-      trans->priv->proportion = 1.0;
-      trans->priv->earliest_time = -1;
-      GST_OBJECT_UNLOCK (trans);
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      break;
-    default:
-      break;
-  }
-
-  result = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      trans->have_same_caps = FALSE;
-      /* We can only reset the passthrough mode if the instance told us to 
-         handle it in configure_caps */
-      if (bclass->passthrough_on_same_caps) {
-        gst_base_transform_set_passthrough (trans, FALSE);
-      }
-      gst_caps_replace (&trans->cache_caps1, NULL);
-      gst_caps_replace (&trans->cache_caps2, NULL);
-      if (bclass->stop)
-        result = bclass->stop (trans);
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-
-  return result;
-}
-
 /**
  * gst_base_transform_set_passthrough:
  * @trans: the #GstBaseTransform to set
