@@ -35,6 +35,7 @@
 #include "../../gst/gst-i18n-lib.h"
 
 #include <gst/gst.h>
+#include <stdio.h>              /* for fseeko() */
 #include <errno.h>
 #include "gstfilesink.h"
 #include <string.h>
@@ -44,7 +45,6 @@
 #include <unistd.h>
 #endif
 
-
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -542,7 +542,28 @@ gst_file_sink_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return FALSE;
   }
   g_free (protocol);
-  location = gst_uri_get_location (uri);
+
+  /* allow file://localhost/foo/bar by stripping localhost but fail
+   * for every other hostname */
+  if (g_str_has_prefix (uri, "file://localhost/")) {
+    char *tmp;
+
+    /* 16 == strlen ("file://localhost") */
+    tmp = g_strconcat ("file://", uri + 16, NULL);
+    /* we use gst_uri_get_location() although we already have the
+     * "location" with uri + 16 because it provides unescaping */
+    location = gst_uri_get_location (tmp);
+    g_free (tmp);
+  } else {
+    location = gst_uri_get_location (uri);
+    if (!location)
+      return FALSE;
+    if (!g_path_is_absolute (location)) {
+      g_free (location);
+      return FALSE;
+    }
+  }
+
   ret = gst_file_sink_set_location (sink, location);
   g_free (location);
 
