@@ -788,8 +788,8 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
   if (G_UNLIKELY (ret < 0))
     goto could_not_read;
 
-  /* regular files should have given us what we expected */
-  if (G_UNLIKELY ((guint) ret < length && src->is_regular))
+  /* seekable regular files should have given us what we expected */
+  if (G_UNLIKELY ((guint) ret < length && src->seekable))
     goto unexpected_eos;
 
   /* other files should eos if they read 0 and more was requested */
@@ -943,7 +943,7 @@ gst_file_src_start (GstBaseSrc * basesrc)
   {
     /* If not in mmap mode, we need to check if the underlying file is
      * seekable. */
-    off_t res = lseek (src->fd, 0, SEEK_CUR);
+    off_t res = lseek (src->fd, 0, SEEK_END);
 
     if (res < 0) {
       GST_LOG_OBJECT (src, "disabling seeking, not in mmap mode and lseek "
@@ -952,6 +952,7 @@ gst_file_src_start (GstBaseSrc * basesrc)
     } else {
       src->seekable = TRUE;
     }
+    lseek (src->fd, 0, SEEK_SET);
   }
 
   /* We can only really do seeking on regular files - for other file types, we
@@ -1061,7 +1062,28 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return FALSE;
   }
   g_free (protocol);
-  location = gst_uri_get_location (uri);
+
+  /* allow file://localhost/foo/bar by stripping localhost but fail
+   * for every other hostname */
+  if (g_str_has_prefix (uri, "file://localhost/")) {
+    char *tmp;
+
+    /* 16 == strlen ("file://localhost") */
+    tmp = g_strconcat ("file://", uri + 16, NULL);
+    /* we use gst_uri_get_location() although we already have the
+     * "location" with uri + 16 because it provides unescaping */
+    location = gst_uri_get_location (tmp);
+    g_free (tmp);
+  } else {
+    location = gst_uri_get_location (uri);
+    if (!location)
+      return FALSE;
+    if (!g_path_is_absolute (location)) {
+      g_free (location);
+      return FALSE;
+    }
+  }
+
   ret = gst_file_src_set_location (src, location);
   g_free (location);
 
