@@ -286,7 +286,9 @@ GST_START_TEST (test_message_state_changed_children)
    * preroll */
   ASSERT_OBJECT_REFCOUNT (src, "src", 3);
   ASSERT_OBJECT_REFCOUNT (sink, "sink", 3);
-  ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline", 2);
+  /* 2 or 3 is valid, because the pipeline might still be posting 
+   * its state_change message */
+  ASSERT_OBJECT_REFCOUNT_BETWEEN (pipeline, "pipeline", 2, 3);
 
   pop_messages (bus, 3);
   fail_if ((gst_bus_pop (bus)) != NULL);
@@ -686,9 +688,11 @@ GST_START_TEST (test_children_state_change_order_semi_sink)
   pop_messages (bus, 4);        /* pop playing => paused messages off the bus */
   pop_messages (bus, 4);        /* pop paused => ready messages off the bus */
 
+  GST_DEBUG ("waiting for pipeline to reach refcount 1");
   while (GST_OBJECT_REFCOUNT_VALUE (pipeline) > 1)
     THREAD_SWITCH ();
 
+  GST_DEBUG ("checking refcount");
   ASSERT_OBJECT_REFCOUNT (src, "src", 1);
   ASSERT_OBJECT_REFCOUNT (sink, "sink", 1);
   ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline", 1);
@@ -696,10 +700,12 @@ GST_START_TEST (test_children_state_change_order_semi_sink)
   ret = gst_element_set_state (pipeline, GST_STATE_NULL);
   fail_if (ret != GST_STATE_CHANGE_SUCCESS, "State change to NULL failed");
 
+  GST_DEBUG ("checking refcount");
   ASSERT_OBJECT_REFCOUNT (src, "src", 1);
   ASSERT_OBJECT_REFCOUNT (sink, "sink", 1);
   ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline", 1);
 
+  GST_DEBUG ("cleanup");
   gst_object_unref (bus);
   gst_object_unref (pipeline);
 }
