@@ -775,6 +775,9 @@ main (int argc, char *argv[])
 
   setlocale (LC_ALL, "");
 
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
+
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
